unit FMain;

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, IniFiles, ActnList, Buttons, Contnrs;

type
  TFormMain = class(TForm)
    StatusBar1: TStatusBar;
    pnlSuperior: TPanel;
    Label1: TLabel;
    edtConn: TEdit;
    Button1: TBitBtn;
    Label2: TLabel;
    mmSQL: TMemo;
    pnlCentral: TPanel;
    pnlInferior: TPanel;
    mmLog: TMemo;
    Button2: TBitBtn;
    Panel1: TPanel;
    Label3: TLabel;
    Image1: TImage;
    Splitter1: TSplitter;
    btnSINThreads: TBitBtn;
    ActionList1: TActionList;
    ActionProbarConexion: TAction;
    ActionLanzarConsultas: TAction;
    btnCONThreads: TBitBtn;
    ActionLanzarSinThreads: TAction;
    ActionLanzarConThreads: TAction;
    cbTotal: TCheckBox;
    cbDatos: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure edtConnChange(Sender: TObject);
    procedure ActionProbarConexionExecute(Sender: TObject);
    procedure ActionLanzarConsultasExecute(Sender: TObject);
    procedure ActionLanzarSinThreadsExecute(Sender: TObject);
    procedure ActionLanzarConThreadsExecute(Sender: TObject);
    procedure StatusBar1Click(Sender: TObject);
    procedure cbTotalClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    SaveFile:TInifile;
    path:string;
    _ini,_fin:Double; // tiempo total
    numThreads:integer;  // Numero de threads activos
    ThreadList:TObjectList;

    procedure _Log(AStr:string);

    procedure LanzarSQL(AConnection, ASQL:String);
    procedure LanzarSQLThread(AConnection, ASQL:String);

    procedure TerminateThread(Sender: TObject);

  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;


//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

{$R *.dfm}

uses
  FData, ShellAPI, UADOSQLThread, ADODB;


procedure TFormMain._Log(AStr: string);
var
  Str:String;
begin
  Str := FormatDateTime('hh:nn:ss:zzzz', Now());
  mmLog.Lines.Add(Str + '  -> ' + AStr);

end;

procedure TFormMain.Button1Click(Sender: TObject);
begin
  // conexin
  edtConn.Text := PromptDataSource(Self.Handle, edtConn.Text);
end;

procedure TFormMain.FormShow(Sender: TObject);
var
  TS:TStrings;
  Str:string;
  i:Integer;
begin

  // ini
  Randomize;

  path := ChangeFileExt(Application.ExeName, '.ini');
  SaveFile := TInifile.Create(path);
  // leer la conexin
  Str := SaveFile.ReadString('Connection', 'ConnString', '');
  edtConn.Text := Str;
  _Log('Recuperada cadena de conexin.');
  // Leer Sentencias SQL
  TS := TStringList.Create();
  try
    SaveFile.ReadSectionValues('SQL', TS);
    // Extraer las sentencias
    for i := 0 to (TS.Count - 1) do begin
      Str := TS.Values[TS.Names[i]];
      mmSQL.Lines.Add(Str);
    end;
    _Log('Resuparadas sentencias SQL.');
  finally
    FreeAndNil(TS);
  end;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Str:string;
  i:Integer;
begin
  // Lipiamos pra escribir las nuevas
  SaveFile.EraseSection('SQL');
  // escribir la conexin
  SaveFile.WriteString('Connection', 'ConnString', edtConn.Text);
  _Log('Grabada cadena de conexin.');
  // Grabar las sentencias
  for i := 0 to (mmSQL.Lines.Count - 1) do begin
    // Escribir las Sentencias SQL
    SaveFile.WriteString('SQL', IntToStr(i), mmSQL.Lines[i]);
  end;
  _Log('Grabadas sentencias SQL.');
  // liberar
  FreeAndNil(SaveFile);

  // Liberar la lista
  FreeAndNil(ThreadList);

  _Log('Cerrar programa.');
end;

procedure TFormMain.edtConnChange(Sender: TObject);
begin
  edtConn.Color := clWhite;
end;

procedure TFormMain.ActionProbarConexionExecute(Sender: TObject);
var
  ADOConn:TADOConnection;
begin

  // Testear la conexin
  ADOConn := TADOConnection.Create(nil);
  // Try para liberar
  try
    ADOConn.ConnectionObject.ConnectionString := edtConn.Text;
    ADOConn.LoginPrompt := False;
    // probar
    try
      ADOConn.Connected := True;
      edtConn.Color := clMoneyGreen;
      _Log('La conexin se ha realizado con xito.');
    except
      // Nada
      on E:Exception do begin
        MessageDlg('Se ha producido un error al intentar conectar. '+#13#10+
                   E.Message, mtWarning, [mbOK], 0);
        _Log('Error al conectar. ' + E.Message);
        edtConn.Color := $00BBBBFF;
      end;
    end;

    // Activar las Acciones
    ActionLanzarSinThreads.Enabled := ADOConn.Connected;
    ActionLanzarConThreads.Enabled := ADOConn.Connected;

    // cerrar
    ADOConn.Close;

  finally
    FreeAndNil(ADOConn);
  end;

end;

procedure TFormMain.ActionLanzarConsultasExecute(Sender: TObject);
var
  TS:TStrings;
  i:Integer;
  Str, SQL:String;
begin


  // Leer Sentencias SQL
  TS := TStringList.Create();
  try

    numThreads := 0;
    TS.Text := mmSQL.Lines.Text;
    SQL := '';
    Screen.Cursor := crSQLWait;

    _ini := now;
    for i := 0 to (TS.Count - 1) do begin

      Str := Trim(TS[i]);

      if (Str <> '') then begin
        if (Str[1] = '@') then begin
          Str := TS[i];  // mantener espacios

          if (SQL <> '') then begin

            // EJECUTAR ************************************
            if (Sender = btnSINThreads) then begin
              // Lanzar la ltima
              LanzarSQL(Self.edtConn.Text, SQL);
            end
            else begin
              // utilizar Threads
              LanzarSQLThread(Self.edtConn.Text, SQL);
            end;
            // *********************************************

            SQL := '';
          end;

          // Inicio de la siguiente
          SQL := Copy(Str, 2, Length(Str));
        end
        else begin
          SQL := SQL + Copy(Str, 1, Length(Str));
        end;
      end;

    end;

    // Lanzar la ltima
    // EJECUTAR ************************************
    if (Sender = btnSINThreads) then begin
      // Lanzar la ltima
      LanzarSQL(Self.edtConn.Text, SQL);

      _fin := now;

      // Tiempo total
      Str := FormatDateTime('nn:ss:zzz', _fin-_ini);
      _Log('');
      _Log('Tiempo total(todo): ' + Str);
      _Log('');

      FormMain.BringToFront;

    end
    else begin
      // utilizar Threads
      LanzarSQLThread(Self.edtConn.Text, SQL);
    end;
    // *********************************************
    Screen.Cursor := crDefault;

  finally
    FreeAndNil(TS);
  end;

end;

procedure TFormMain.LanzarSQL(AConnection, ASQL: String);
var
  ADOQ:TADOQuery;
  ini,fin:Double;
  str:string;
  F:TFormData;
begin

  ini := Now;
  //_Log('Lanzando consulta...');
  //_Log('   ' + ASQL);
  //crea el query   mssql
  ADOQ := TADOQuery.Create(nil);

  // Para liberar
  try
    ADOQ.ConnectionString := AConnection;
    ADOQ.SQL.Add(ASQL);

    // Lanzarlo
    ADOQ.Open;
    Application.ProcessMessages;
    //_Log('   Query abierta');

    Application.ProcessMessages;
    fin := now;

    // Tiempos totales slo?
    if not (cbTotal.Checked) then begin
      Str := FormatDateTime('nn:ss:zzz', fin-ini);
      _Log('          ' + ASQL);
      _Log('            ' + Str);
    end;
  finally
    // Mostrar Datos
    if (cbDatos.Checked) then begin
      // Crear el form de Datos
      F := TFormData.Create(Application);
      F.DS := ADOQ;
      // Visualizar
      F.Show;
    end
    else begin
      FreeAndNil(ADOQ);
    end;
  end;



  //_Log('Visualizacin finalizada...');

end;

procedure TFormMain.LanzarSQLThread(AConnection, ASQL: String);
var
  ini,fin:Double;
  str:string;
  th:TADOSQLThread;
begin

  // ini
  inc(numThreads);
  ini := Now;
  //_Log('Lanzando consulta con thread...');
  //_Log('   ' + ASQL);

  //crear el Thread
  th := TADOSQLThread.Create(True, AConnection, ASQL);

  // Aadirlo a la lista
  ThreadList.Add(th);

  //_Log('   Creado el thread...');
  //-th.TimeSQL := now();
  // evento finalizacion
  th.OnTerminate := TerminateThread;

  // Ejecutarlo
  th.Resume;

end;

procedure TFormMain.ActionLanzarSinThreadsExecute(Sender: TObject);
begin
  // Dsactivar
  ActionLanzarSinThreads.Enabled := False;
  ActionLanzarConThreads.Enabled := False;
  try
    // Lanzar
    ActionLanzarConsultasExecute(btnSINThreads);
  finally
    // Activar
    ActionLanzarSinThreads.Enabled := True;
    ActionLanzarConThreads.Enabled := True;
  end;
end;

procedure TFormMain.ActionLanzarConThreadsExecute(Sender: TObject);
begin
  //Desactivar
  ActionLanzarSinThreads.Enabled := False;
  ActionLanzarConThreads.Enabled := False;
  // Lanzar
  ActionLanzarConsultasExecute(btnCONThreads);
end;

procedure TFormMain.TerminateThread(Sender: TObject);
var
  str:string;
  th:TADOSQLThread;
  timeSQL:TDateTime;
  F:TFormData;
begin

  // Thread
  th := TADOSQLThread(Sender);
  // decrementar activos
  dec(numThreads);
  StatusBar1.Panels[1].Text := IntToStr(numThreads) + ' threads activos';
  _Log(IntToStr(numThreads) + ' threads activos');

  // Tiempos totales slo?
  if not (cbTotal.Checked) then begin
    _Log('          ' + th.SQL);
    _Log('            ' + th.TotalTime);
  end;

  // Mostrar Datos
  if (cbDatos.Checked) then begin
    // Crear el form de Datos
    F := TFormData.Create(Application);
    F.DS := th.ADOQ;
    // Visualizar
    F.Show;
  end;


  // Han terminado todos
  if (numThreads = 0) then begin;

    _fin := now;

    // Tiempo total
    Str := FormatDateTime('nn:ss:zzz', _fin-_ini);
    _Log('');
    _Log('Tiempo total con threads(todo): ' + Str);
    _Log('');

    FormMain.BringToFront;
    Screen.Cursor := crDefault;

    // Activar
    ActionLanzarSinThreads.Enabled := True;
    ActionLanzarConThreads.Enabled := True;

    // Limpiar los threads de la lista de objetos
    //ThreadList.Clear;
  end;


end;

procedure TFormMain.StatusBar1Click(Sender: TObject);
begin
  ShellExecute(Handle, 'open', 'http://neftali.clubdelphi.com/', 0, 0, 0);
end;

procedure TFormMain.cbTotalClick(Sender: TObject);
begin
  // no modificarlo durante la ejecucin
  cbTotal.Enabled := False;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  // Crear la lista de threads
  ThreadList := TObjectList.Create();
  ThreadList.OwnsObjects := False;
end;

end.
